#ifndef SUFFIXARRAY_H
#define SUFFIXARRAY_H

#include "misc/definitions.h"


namespace CSA
{


const std::string SA_EXTENSION = ".sa";


class SuffixArray
{
  public:

//--------------------------------------------------------------------------
//  CONSTRUCTION
//--------------------------------------------------------------------------

    SuffixArray(const std::string& base_name, bool print = false);
    // Data will be deleted with the suffix array.
    SuffixArray(uchar* _data, uint bytes, uint threads = 1);
    ~SuffixArray();

    void writeTo(const std::string& base_name, bool write_data = false) const;

    inline bool isOk() const { return this->ok; }

//--------------------------------------------------------------------------
//  QUERIES
//--------------------------------------------------------------------------

    // Returns the closed range containing the matches.
    pair_type count(const std::string& pattern) const;

    // User must free the array.
    uint* locate(pair_type range) const;

    // Returns SA[index].
    uint locate(uint index) const;
    
//--------------------------------------------------------------------------
//  REPORTING
//--------------------------------------------------------------------------

    inline uint getSize() const { return this->data_size; }
    inline uint getNumberOfSequences() const { return this->sequences; }

    // Returns the size of the data structure.
    usint reportSize(bool print = false) const;

//--------------------------------------------------------------------------
//  INTERNAL VARIABLES
//--------------------------------------------------------------------------

  private:
    bool ok;

    uchar* data;
    uint*  sa;
    uint   data_size;
    uint   sequences;

//--------------------------------------------------------------------------
//  INTERNAL STUFF
//--------------------------------------------------------------------------

    uint match(const std::string& pattern, uint index, uint matched) const;

    // These are not allowed.
    SuffixArray();
    SuffixArray(const SuffixArray&);
    SuffixArray& operator = (const SuffixArray&);
};


} // namespace CSA


#endif // SUFFIXARRAY_H
